@@ -52,6 +52,7 @@ class MultiEditViewActivatable(GObject.Object, Gedit.ViewActivatable, Signals):
         self._status = None
         self._status_timeout = 0
         self._delete_mode_id = 0
+        self._tooltip_widget = None
 
     def do_activate(self):
         self.view.multiedit_view_activatable = self
@@ -72,6 +73,7 @@ class MultiEditViewActivatable(GObject.Object, Gedit.ViewActivatable, Signals):
 
         self.connect_signals(self.view, handlers)
         self.connect_signal_after(self.view, 'move-cursor', self.on_move_cursor_after)
+        self.connect_signal_after(self.view, 'draw', self.on_view_draw_after)
 
         self.view.props.has_tooltip = True
 
@@ -143,18 +145,26 @@ class MultiEditViewActivatable(GObject.Object, Gedit.ViewActivatable, Signals):
         self._buffer = newbuf
 
     def initialize_event_handlers(self):
-        self._event_handlers = [
-            [('Escape',), 0, self.do_escape_mode, True],
-            [('Return',), 0, self.do_column_edit, True],
-            [('Return',), Gdk.ModifierType.CONTROL_MASK, self.do_smart_column_edit, True],
-            [('Return',), Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK, self.do_smart_column_align, True],
-            [('Return',), Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK | Gdk.ModifierType.MOD1_MASK, self.do_smart_column_align, True],
-            [('Home',), Gdk.ModifierType.CONTROL_MASK, self.do_mark_start, True],
-            [('End',), Gdk.ModifierType.CONTROL_MASK, self.do_mark_end, True],
-            [('e', 'E'), Gdk.ModifierType.CONTROL_MASK, self.do_toggle_edit_point, True]
-        ]
+        keymap = Gdk.Keymap.get_for_display(self.view.get_display())
 
-        for handler in self._event_handlers:
+        modmask = keymap.get_modifier_mask(Gdk.ModifierIntent.PRIMARY_ACCELERATOR)
+        withvirt = keymap.add_virtual_modifiers(modmask)
+
+        if modmask != withvirt:
+            modmask = Gdk.ModifierType(withvirt & ~modmask)
+
+        self._event_handlers = {
+            'escape_mode': [('Escape',), 0, self.do_escape_mode, True],
+            'column_edit': [('Return',), 0, self.do_column_edit, True],
+            'smart_column_edit': [('Return',), modmask, self.do_smart_column_edit, True],
+            'smart_column_align': [('Return',), modmask | Gdk.ModifierType.SHIFT_MASK, self.do_smart_column_align, True],
+            'smart_column_align_space': [('Return',), modmask | Gdk.ModifierType.SHIFT_MASK | Gdk.ModifierType.MOD1_MASK, self.do_smart_column_align, True],
+            'mark_start': [('Home',), modmask, self.do_mark_start, True],
+            'mark_end': [('End',), modmask, self.do_mark_end, True],
+            'toggle_edit_point': [('E', 'e'), modmask, self.do_toggle_edit_point, True]
+        }
+
+        for handler in self._event_handlers.values():
             handler[0] = list(map(lambda x: Gdk.keyval_from_name(x), handler[0]))
 
     def get_window_activatable(self):
@@ -769,10 +779,13 @@ class MultiEditViewActivatable(GObject.Object, Gedit.ViewActivatable, Signals):
         return True
 
     def on_key_press_event(self, view, event):
+        keymap = Gdk.Keymap.get_for_display(view.get_display())
         defmod = Gtk.accelerator_get_default_mod_mask() & event.state
 
-        for handler in self._event_handlers:
-            if (not handler[3] or self._in_mode) and event.keyval in handler[0] and (defmod == handler[1]):
+        for handler in self._event_handlers.values():
+            _, state = keymap.map_virtual_modifiers(handler[1])
+
+            if (not handler[3] or self._in_mode) and event.keyval in handler[0] and (defmod == state):
                 return handler[2](event)
 
         return False
@@ -1232,49 +1245,77 @@ class MultiEditViewActivatable(GObject.Object, Gedit.ViewActivatable, Signals):
 
         return lbl
 
+    def event_handler_accel_name(self, name):
+        handler = self._event_handlers[name]
+
+        keyval = handler[0][0]
+        modmask = handler[1]
+
+        return Gtk.accelerator_name(keyval, modmask)
+
     def on_query_tooltip(self, view, x, y, keyboard_mode, tooltip):
         if not self._in_mode:
             return False
 
-        geom = view.get_window(Gtk.TextWindowType.TOP).get_geometry()
+        win = view.get_window(Gtk.TextWindowType.TOP)
+        parent = win.get_parent()
+
+        while view.get_window_type(parent) != Gtk.TextWindowType.WIDGET:
+            parent = parent.get_parent()
+
+        wino = win.get_origin()
+        paro = parent.get_origin()
+
+        geom = (wino[1] - paro[1], wino[2] - paro[2], win.get_width(), win.get_height())
 
         if x < geom[0] or x > geom[0] + geom[2] or y < geom[1] or y > geom[1] + geom[3]:
             return False
 
-        grid = Gtk.Grid()
-        grid.set_row_spacing(3)
-        grid.set_column_spacing(12)
+        if self._tooltip_widget is None:
+            grid = Gtk.Grid()
+            grid.set_row_spacing(3)
+            grid.set_column_spacing(12)
 
-        grid.attach(self.make_label('<b>Selection</b>', True), 0, 0, 1, 1)
-        grid.attach(self.make_label('<Enter>', False), 0, 1, 1, 1)
-        grid.attach(self.make_label('<Ctrl><Enter>', False), 0, 2, 1, 1)
-        grid.attach(self.make_label('<Ctrl><Shift><Enter>', False), 0, 3, 1, 1)
-        grid.attach(self.make_label('<Ctrl><Alt><Shift><Enter>', False), 0, 4, 1, 1)
+            self._event_handlers['column_edit']
+            self._event_handlers['smart_column_edit']
+            self._event_handlers['smart_column_align']
+            self._event_handlers['smart_column_align_space']
+            self._event_handlers['mark_start']
+            self._event_handlers['mark_end']
+            self._event_handlers['toggle_edit_point']
 
-        sep = Gtk.Separator.new(Gtk.Orientation.HORIZONTAL)
-        sep.show()
+            grid.attach(self.make_label('<b>Selection</b>', True), 0, 0, 1, 1)
+            grid.attach(self.make_label(self.event_handler_accel_name('column_edit'), False), 0, 1, 1, 1)
+            grid.attach(self.make_label(self.event_handler_accel_name('smart_column_edit'), False), 0, 2, 1, 1)
+            grid.attach(self.make_label(self.event_handler_accel_name('smart_column_align'), False), 0, 3, 1, 1)
+            grid.attach(self.make_label(self.event_handler_accel_name('smart_column_align_space'), False), 0, 4, 1, 1)
 
-        grid.attach(sep, 0, 5, 2, 1)
-        grid.attach(self.make_label('<b>Edit points</b>', True), 0, 6, 1, 1)
-        grid.attach(self.make_label('<Ctrl>+E', False), 0, 7, 1, 1)
-        grid.attach(self.make_label('<Ctrl><Home>', False), 0, 8, 1, 1)
-        grid.attach(self.make_label('<Ctrl><End>', False), 0, 9, 1, 1)
-        grid.attach(self.make_label('<Ctrl><Shift><Enter>', False), 0, 10, 1, 1)
-        grid.attach(self.make_label('<Ctrl><Alt><Shift><Enter>', False), 0, 11, 1, 1)
+            sep = Gtk.Separator.new(Gtk.Orientation.HORIZONTAL)
+            sep.show()
 
-        grid.attach(self.make_label(_('Enter column edit mode using selection')), 1, 1, 1, 1)
-        grid.attach(self.make_label(_('Enter <b>smart</b> column edit mode using selection')), 1, 2, 1, 1)
-        grid.attach(self.make_label(_('<b>Smart</b> column align mode using selection')), 1, 3, 1, 1)
-        grid.attach(self.make_label(_('<b>Smart</b> column align mode with additional space using selection')), 1, 4, 1, 1)
+            grid.attach(sep, 0, 5, 2, 1)
+            grid.attach(self.make_label('<b>Edit points</b>', True), 0, 6, 1, 1)
+            grid.attach(self.make_label(self.event_handler_accel_name('toggle_edit_point'), False), 0, 7, 1, 1)
+            grid.attach(self.make_label(self.event_handler_accel_name('mark_start'), False), 0, 8, 1, 1)
+            grid.attach(self.make_label(self.event_handler_accel_name('mark_end'), False), 0, 9, 1, 1)
+            grid.attach(self.make_label(self.event_handler_accel_name('smart_column_align'), False), 0, 10, 1, 1)
+            grid.attach(self.make_label(self.event_handler_accel_name('smart_column_align_space'), False), 0, 11, 1, 1)
 
-        grid.attach(self.make_label(_('Toggle edit point')), 1, 7, 1, 1)
-        grid.attach(self.make_label(_('Add edit point at beginning of line/selection')), 1, 8, 1, 1)
-        grid.attach(self.make_label(_('Add edit point at end of line/selection')), 1, 9, 1, 1)
-        grid.attach(self.make_label(_('Align edit points')), 1, 10, 1, 1)
-        grid.attach(self.make_label(_('Align edit points with additional space')), 1, 11, 1, 1)
+            grid.attach(self.make_label(_('Enter column edit mode using selection')), 1, 1, 1, 1)
+            grid.attach(self.make_label(_('Enter <b>smart</b> column edit mode using selection')), 1, 2, 1, 1)
+            grid.attach(self.make_label(_('<b>Smart</b> column align mode using selection')), 1, 3, 1, 1)
+            grid.attach(self.make_label(_('<b>Smart</b> column align mode with additional space using selection')), 1, 4, 1, 1)
 
-        grid.show_all()
-        tooltip.set_custom(grid)
+            grid.attach(self.make_label(_('Toggle edit point')), 1, 7, 1, 1)
+            grid.attach(self.make_label(_('Add edit point at beginning of line/selection')), 1, 8, 1, 1)
+            grid.attach(self.make_label(_('Add edit point at end of line/selection')), 1, 9, 1, 1)
+            grid.attach(self.make_label(_('Align edit points')), 1, 10, 1, 1)
+            grid.attach(self.make_label(_('Align edit points with additional space')), 1, 11, 1, 1)
+
+            grid.show_all()
+            self._tooltip_widget = grid
+
+        tooltip.set_custom(self._tooltip_widget)
         return True
 
     def get_border_color(self):
@@ -1291,9 +1332,12 @@ class MultiEditViewActivatable(GObject.Object, Gedit.ViewActivatable, Signals):
     def on_view_draw(self, view, cr):
         window = view.get_window(Gtk.TextWindowType.TEXT)
 
-        if Gtk.cairo_should_draw_window (cr, window):
-            return self._draw_column_mode(cr)
+        if not window is None and Gtk.cairo_should_draw_window(cr, window):
+            self._draw_column_mode(cr)
 
+        return False
+
+    def on_view_draw_after(self, view, cr):
         window = view.get_window(Gtk.TextWindowType.TOP)
 
         if window is None or not Gtk.cairo_should_draw_window(cr, window):
@@ -1310,6 +1354,8 @@ class MultiEditViewActivatable(GObject.Object, Gedit.ViewActivatable, Signals):
 
         Gtk.cairo_transform_to_window(cr, view, window)
 
+        cr.save()
+
         cr.translate(0.5, 0.5)
         cr.set_line_width(1)
 
@@ -1319,11 +1365,15 @@ class MultiEditViewActivatable(GObject.Object, Gedit.ViewActivatable, Signals):
         cr.move_to(0, h - 1)
         cr.rel_line_to(w, 0)
         cr.stroke()
+        cr.restore()
 
-        context = self.view.get_style_context()
-        Gdk.cairo_set_source_rgba(cr, context.get_color(Gtk.StateFlags.NORMAL))
-        cr.move_to(w - extents[1].width - 3, (h - extents[1].height) / 2)
-        PangoCairo.show_layout(cr, layout)
+        ctx = self.view.get_style_context()
+        ctx.save()
+        ctx.add_class('top')
+
+        cr.save()
+        Gtk.render_layout(ctx, cr, w - extents[1].width - 3, (h - extents[1].height) / 2, layout)
+        cr.restore()
 
         if not self._status:
             status = ''
@@ -1333,8 +1383,11 @@ class MultiEditViewActivatable(GObject.Object, Gedit.ViewActivatable, Signals):
         if status:
             layout.set_markup(status, -1)
 
-            cr.move_to(3, (h - extents[1].height) / 2)
-            PangoCairo.show_layout(cr, layout)
+            cr.save()
+            Gtk.render_layout(ctx, cr, 3, (h - extents[1].height) / 2, layout)
+            cr.restore()
+
+        ctx.restore()
 
         return False
 
